///////////////////////////////////////////////////
///	Title   : Clay's Notepad                //
///     Author  : Clay Hanson (ID: 15144)      //
///     BL Rev  : r1985                       //
///	Font    : Consolas 11 Regular        //
//////////////////////////////////////////////
///	    TIMELINE                        ||
/// 9/08/2016 First official release        ||
/// 9/20/2016 3.6.0 release                 ||
///                                         ||
///                                         ||
//////////////////////////////////////////////


// Executing script files & creating necessary script objects
exec("./profiles.cs");
if(!isObject(NotepadUndo)) new ScriptObject(NotepadUndo);
if(!isObject(NotepadDocGroup)) new ScriptGroup(NotepadDocGroup);
if(!isObject(notepad)) exec("./notepad.gui");
if(!isObject(notepadThemeGui)) exec("./notepadThemeGui.gui");

// Setting variables
$NotepadVersion = "3.7.0";
$CACHE::Notepad::Changelog = "-- Version "@$NotepadVersion@" Changelog --\n\n";
$CACHE::Notepad::Changelog = $CACHE::Notepad::Changelog @ "  + Fixed updater closing after entering the main menu for the first time\n\n";
$CACHE::Notepad::Changelog = $CACHE::Notepad::Changelog @ "  + You can now type 'notepad' in console to open notepad.\n\n";
$CACHE::Notepad::Changelog = $CACHE::Notepad::Changelog @ "  + Added ability to change notepad's current font\n\n";
$CACHE::Notepad::Changelog = $CACHE::Notepad::Changelog @ "  + Accidently clicking outside of the current active window no longer hides the active window underneath the notepad window.\n\n";
$CACHE::Notepad::Date = "12:21 PM 10/18/2016";
$Notepad::About::DemoModeFormat = "(<font:Arial Bold:14><color:ff0000>Demo Mode<font:Arial:14><color:000000>)";
$Notepad::About = "<bitmap:add-ons/system_notepad/notepadlogo><br><br><br><br><br><bitmap:add-ons/system_notepad/devider><just:left><br>Clay's Notepad [[DEMO]]<br>v<font:Arial Bold:14>"@$NotepadVersion@"<font:Arial:14> ("@$CACHE::Notepad::Date@")<br>Made by Clay Hanson [15144]";

// Packaging functions
package Notepad {
	function GuiWindowCtrl::setVisible(%this,%bool) {
		Parent::setVisible(%this,%bool);
		if(!Notepad.isAwake()) return;
		if(stripos(%this.getName(),"notepad") == -1) return;
		%screenSize = Canvas.getExtent();
		%showBlocker = 0;
		for(%i=0;%i<Notepad.getCount();%i++) {
			%obj = Notepad.getObject(%i);
			if(striPos(%obj.getName(),"Notepad") != 0) continue;
			if(%obj.getName() $= "Notepad_Win") continue;
			if(%obj.getName() $= "Notepad_Blocker") continue;
			if(!%obj.isVisible()) continue;
			echo(%obj.getName());
			%showBlocker = 1;
		}
		Notepad_Blocker.setVisible(%showBlocker);
		Notepad_Blocker.resize(0,0,getWord(%screenSize,0),getWord(%screenSize,1));
		if(!%showBlocker) return;
		Notepad.bringToFront(Notepad_Blocker);
		Notepad.bringToFront(Notepad_Win);
	}
	function ConsoleEntry::eval(%this) {
		%text = ConsoleEntry.getValue();
		if(%text $= "notepad") {
			if(!$Pref::Client::Notepad::AllowNotepadConsole) {	
				echo("\c2Preference error in input.");
				echo("\c2eval error >> "@%text);
				backTrace();
				ConsoleEntry.setValue("");
				return;
			}
			notepad();
			echo("==>"@%text);
			ConsoleEntry.setValue("");
			return;
		}
		return Parent::eval(%this);
	}
	function NMH_Type::send(%this) {
		%txt = %this.getvalue();
		if($Pref::Client::Notepad::AllowNotepadCMD && %txt $= "/notepad") {
			canvas.popDialog(newMessageHud);
			notepad();
			return;
		}
		Parent::send(%this);
	}
	function MM_AuthBar::blinkSuccess(%this) {
		if(!$CACHE::InitNotepad) {
			initNotepadThemes();
			NotepadThemes_initLoader();
			$CACHE::InitNotepad = 1;
			$CACHE::NotepadTheme = 2;
			$CACHE::NotepadThemeText = "NotepadWindowProfile";
			Notepad_setTheme("Default");
			initNotepadEscapeMenuButton();
		}
		return Parent::blinkSuccess(%this);
	}
	function messageBoxNotepadOkDlg::onSleep(%this) {
		if(isFunction(messageBoxNotepadOkDlg,onSleep)) Parent::onSleep(%this);
		%a = MBOKFrame;
		if(%a.profile !$= "blockWindowProfile") %a.setProfile("blockWindowProfile");
	}
	function Notepad::onSleep(%this) {
		cancel(Notepad.schedulethis);
	}
	function Notepad_FS_L::clear(%this) {
		Notepad_FS_L.lastselected = "";
		parent::clear(%this);
	}
	function Notepad::onWake(%this) {
		%this.doOnWakeFunctions();
	}
	function Notepad::doOnWakeFunctions(%this) {
		Notepad_Blocker.setVisible(0);
		Notepad_saveUndo();
		Notepad_Win.centerY();
		Notepad_Win.centerX();
		%menu = Notepad_M;
		%menu.clearMenus();
		%menu.addMenu("File",0);
		%menu.addMenuItem("File","New",1,"Ctrl N",1);
		%menu.addMenuItem("File","Open...",2,"Ctrl O",1);
		%menu.addMenuItem("File","Save",3,"Ctrl S",1);
		%menu.addMenuItem("File","Save As...",4,"",1);
		%menu.addMenuItem("File","-",0);
		%menu.addMenuItem("File","- Recent Files -",5);
		%menu.setMenuItemEnable("File",5,0);
		// Recent Files
		%menuCnt = 5;
		%recentFiles = trim($CACHE::Notepad::RecentFiles);
		if(%recentFiles !$= "") {
			for(%i=0;%i<getRecordCount(%recentFiles);%i++) {
				%rec = getRecord(%recentFiles,%i);
				if(%rec $= "") continue;
				if(!isFile(%rec)) continue;
				%menuCnt++;
				%menu.addMenuItem("File",%rec,%menuCnt,"",1);
			}
		}		
		// Recent Files
		%menu.addMenuItem("File","(Clear Recent List)",%menuCnt+1,"",1);
		%menu.addMenuItem("File","-",0);
		%menu.addMenuItem("File","Exit",%menuCnt+2,"Ctrl F",1);
		%menu.fileClearRecentListID = %menuCnt+1;
		%menu.fileExtID = %menuCnt+2;
		%menu.addMenu("Edit",1);
		%menu.addMenuItem("Edit","Redo",1,"Ctrl Y",1);
		%menu.addMenuItem("Edit","Undo",2,"Ctrl Z",1);
		%menu.addMenuItem("Edit","-",0);
		%menu.addMenuItem("Edit","Cut",3,"Ctrl X",1);
		%menu.addMenuItem("Edit","Copy",4,"Ctrl C",1);
		%menu.addMenuItem("Edit","Paste",5,"Ctrl V",1);
		%menu.addMenuItem("Edit","Delete",6,"Del",1);
		%menu.addMenuItem("Edit","-",0);
		%menu.addMenuItem("Edit","Find...",7,"Ctrl F",1);
		%menu.addMenuItem("Edit","Replace...",8,"Ctrl H",1);
		%menu.addMenuItem("Edit","-",0);
		%menu.addMenuItem("Edit","Options",9,"",1);
		%menu.addMenuItem("Edit","Font...",10,"",1);
		%menu.addMenu("Script",2);
		%menu.addMenuItem("Script","Execute script (server)",1,"",1);
		%menu.addMenuItem("Script","Execute script (client)",2,"",1);
		%menu.addMenu("Themes",3);
		%menu.addMenuItem("Themes","Create Theme...",1,"",2);
		%menu.addMenuItem("Themes","Windows 8",2,"",3);
		%mC = 2;
		for(%i=0;%i<NotepadThemeGroup.getCount();%i++) {
			%obj = NotepadThemeGroup.getObject(%i);
			%n = %obj.notepadName;
			if(%n $= "") %n = strReplace(%obj.getName(),"NotepadWindowTheme","");
			%mC++;
			%menu.addMenuItem("Themes",%n,%mC,"",2);
		}
		%menu.ThemeCount = %mC;
		%menu.addMenu("Document Storage",4);
		%menu.addMenuItem("Document Storage","Save document",1,"",1);
		%menu.addMenuItem("Document Storage","Delete all",2,"",1);
		%menu.addMenuItem("Document Storage","-",0);
		%mC = 2;
		for(%i=0;%i<NotepadDocGroup.getCount();%i++) {
			%dN = NotepadDocGroup.getObject(%i);
			%mC++;
			%menu.addMenuItem("Document Storage",%dN.docName,%mC,%dN.docModifier SPC %dN.docDate,1);
		}
		if(%mC == 2) %menu.addMenuItem("Document Storage","No documents!",0,"",1);
		if(%mC == 2) %menu.setMenuItemEnable("Document Storage","No documents!",0);
		if(%mC == 2) %menu.setMenuItemEnable("Document Storage","Delete all",0);
		%menu.addMenu("Help",5);
		%menu.addMenuItem("Help","Changelog",1,"",1);
		%menu.addMenuItem("Help","-",0);
		%menu.addMenuItem("Help","About Notepad",2,"",1);
		%menu.addMenuItem("Help","Open Support Page",3,"",1);
		// Edit
		%menu.setMenuItemEnable("Edit","Paste",1);
		%menu.setMenuItemEnable("Edit","Cut",1);
		%menu.setMenuItemEnable("Edit","Copy",1);
		%menu.setMenuItemEnable("Edit","Delete",1);
		// Now let's calculate it.
		Notepad_M.setMenuItemChecked("Themes","Windows 8",0);
		%mC = 2;
		for(%i=0;%i<NotepadThemeGroup.getCount();%i++) {
			%obj = NotepadThemeGroup.getObject(%i);
			%n = %obj.notepadName;
			if(%n $= "") %n = strReplace(%obj.getName(),"NotepadWindowTheme","");
			%mC++;
			Notepad_M.setMenuItemChecked("Themes",%mC,0);
		}
		Notepad_M.setMenuItemChecked("Themes",$CACHE::NotepadTheme,1);
		%menu.setMenuItemEnable("Edit","Cut",0);
		%menu.setMenuItemEnable("Edit","Copy",0);
		%menu.setMenuItemEnable("Edit","Delete",1);
		if(getClipboard() $= "") %menu.setMenuItemEnable("Edit","Paste",0);
 
	}
	function Notepad_FS::setVisible(%this,%bool) {
		%this.evaluateVisible(%bool);
		Parent::setVisible(%this,%bool);
	}
	function Notepad_FS::evaluateVisible(%this,%bool) {
		if(%bool) {
			Notepad_FS_P.setText("");
			Notepad.pushToBack(Notepad_FS);
			Notepad_FS_L.clear();
			Notepad_FS_E.setValue("");
			%this.loadFiles("");
		}
		%this.centerY();
		%this.centerX();
	}
};
activatePackage(Notepad);

// Onto everything else
function initNotepadEscapeMenuButton() {
	if(!isObject(escapeMenu))
		return;

	canvas.pushDialog(escapeMenu);
	%window = escapeMenu.getObject(0);
	if(%window.extent $= "221 458")
		return;

	%x = getWord(%window.getPosition(),0);
	%y = getWord(%window.getPosition(),1);
	%window.resize(%x,%y,221,458);
	if(isObject(EM_NotepadBtn))
		EM_NotepadBtn.delete();

	%a = new GuiBitmapButtonCtrl(EM_NotepadBtn) {
		profile = "BlockButtonProfile";
		horizSizing = "center";
		vertSizing = "bottom";
		position = "7 413";
		command = "canvas.popDialog(escapeMenu);notepad();";
		extent = "207 36";
		minExtent = "8 2";
		enabled = "1";
		visible = "1";
		clipToParent = "1";
		text = "Open Notepad";
		groupNum = "-1";
		buttonType = "PushButton";
		bitmap = "base/client/ui/button1";
		lockAspectRatio = "0";
		alignLeft = "0";
		alignTop = "0";
		overflowImage = "0";
		mKeepCached = "0";
		mColor = "100 230 50 255";
		wrap = "0";
	};
	%window.add(%a);
	canvas.popDialog(escapeMenu);
}
function Notepad() {
	if(!$CACHE::initNotepad) {
		error("The game must finish the auth sequence before we can continue");
		return;
	}
	if(!isObject(NotepadThemeGroup)) {
		error("Essential script group 'NotepadThemeGroup' not found; reloading.");
		initNotepadThemes();
		return;
	}
	if(Notepad.isAwake())
		canvas.popDialog(Notepad);
	else
		canvas.pushDialog(Notepad);
}
function Notepad_M::onMenuSelect(%this, %menuID, %menuTxt) {
	%so = NotepadUndo;
	switch(%menuID) {
		case 1:
			%this.setMenuItemEnable("Edit","Redo",1);
			%this.setMenuItemEnable("Edit","Undo",1);
			if(%so.state[%so.undoCount+1] $= "")
				Notepad_M.setMenuItemEnable("Edit","Redo",0);

			if(%so.state[%so.undoCount-1] $= "")
				Notepad_M.setMenuItemEnable("Edit","Undo",0);
		case 2:
			%this.setMenuItemEnable("Script","Execute script (server)",1);
			if(!isObject(ServerConnection) && $CACHE::Notepad::IsAdmin) %this.setMenuItemEnable("Script","Execute script (server)",0);
		case 4:
			%this.setMenuItemEnable("Document Storage",1,0);
			if(Notepad_txt.notepadFile !$= "") %this.setMenuItemEnable("Document Storage",1,1);
	}
}
function Notepad_M::onMenuItemSelect(%this,%menuID,%menuTxt,%menuItemID,%menuItemTxt) {
	switch(%menuID) {
		case 0:
			switch(%menuItemID) {
				case 1:
					messageBoxYesNo("Notepad","Are you sure you want to make a new document?","Notepad_Clear();");
				case 2:
					Notepad.saveorload = "Load";
					Notepad_FS.setVisible(1);
				case 3:
					if(Notepad_Txt.notepadFile !$= "") {
						Notepad_FS_E.setText(fileName(Notepad_Txt.notepadFile));
						Notepad_FS_P.setText(filePath(Notepad_Txt.notepadFile));
						Notepad.saveFile();
					} else {
						Notepad.saveorload = "Save";
						Notepad_FS.setVisible(1);
					}
				case 4:
					Notepad.saveorload = "Save";
					Notepad_FS.setVisible(1);
				case %this.fileExtID:
					canvas.popDialog(Notepad);
				case %this.fileClearRecentListID:
					$CACHE::Notepad::RecentFiles = "";
					canvas.popDialog(Notepad);
					canvas.pushDialog(Notepad);
				default:
					if(!isFile(%menuItemTxt)) return;
					Notepad_FS_E.setValue(fileName(%menuItemTxt));
					Notepad_FS_P.setValue(filePath(%menuItemTxt));
					Notepad.openFile();
			}
		case 1:
			switch(%menuItemID) {
				case 1:
					// Redo
					Notepad_performRedo();
				case 2:
					// Undo
					Notepad_performUndo();
				case 5:
					// Paste
					Notepad_TXT.addText(getClipboard(),1);
				case 7:
					// Find
					Notepad_openToolWindow(0);
				case 8:
					// Replace
					Notepad_openToolWindow(1);
				case 9:
					// Options
					Notepad.openOptionsMenu();
				case 10:
					// Font
					Notepad_initFontWindow();
			}
		case 2:
			switch(%menuItemID) {
				case 1:
					Notepad_sendEval();
				case 2:
					eval(Notepad_txt.getValue());
			}
		case 4:
			// Document Storage
			if(%menuItemID == 1) {
				NotepadStDoc_storeDoc();
				return;
			}
			if(%menuItemID == 2) {
				%cmd = "NotepadDocGroup.deleteAll();canvas.popDialog(Notepad);canvas.pushDialog(Notepad);";
				messageBoxYesNo("Confirmation - Notepad","Are you sure you want to delete all of your saved documents?",%cmd);
				return;
			}
			for(%i=0;%i<NotepadDocGroup.getCount();%i++) {
				%dN = NotepadDocGroup.getObject(%i);
				if(%dN.docName !$= %menuItemTxt) continue;
				NotepadStDoc_getDoc(%dN);
			}
		case 5:
			switch(%menuItemID) {
				case 1:
					messageBoxNotepadOk("Changelog - Notepad",$CACHE::Notepad::Changelog);
				case 2:
					%about = $Notepad::About;
					if(!isUnlocked())
						%about = strReplace(%about,"[[DEMO]]",$Notepad::About::DemoModeFormat);
					else
						%about = strReplace(%about,"[[DEMO]]","");

					messageBoxNotepadOk("About Notepad",%about);
				case 3:
					gotoWebpage("forum.blockland.us");
			}
		case 3:
			if(%menuItemID == 1) {
				// Create Theme...
				canvas.pushDialog(NotepadThemeGui);
				return;
			}
			if(%menuItemID == 2) {
				$CACHE::NotepadTheme = 2;
				Notepad_setTheme("Default");
				$CACHE::NotepadThemeText = "NotepadWindowProfile";
				return;
			}
			%mC = 2;
			for(%i=0;%i<NotepadThemeGroup.getCount();%i++) {
				%obj = NotepadThemeGroup.getObject(%i);
				%mC++;
				%n = %obj.notepadName;
				if(%n $= "") %n = strReplace(%obj.getName(),"NotepadWindowTheme","");
				if(%menuItemTxt !$= %n) continue;
				$CACHE::NotepadTheme = %mC;
				$CACHE::NotepadThemeText = "NotepadWindowTheme"@strReplace(%obj.getName(),"NotepadWindowTheme","");
				Notepad_setTheme(strReplace(%obj.getName(),"NotepadWindowTheme",""));
				return;
			}
	}
}
function Notepad::Done(%this) {
	if(Notepad.saveorload $= "Save")
		%action = Notepad.saveFile();
	else
		Notepad.openFile();

	if(%action $= "") %action = "Load";
	Notepad.saveorload = %action;
}
function Notepad_Clear(%confirm) {
	if(!%confirm) {
		if(Notepad_Txt.notepadFile !$= "") {
			messageBoxYesNo("Warning - Notepad","Do you want to save this file before closing it?","Notepad.saveFile(1,1);","Notepad_Clear(1);");
			return;
		}
	}
	if(isObject(NotepadUndo)) NotepadUndo.delete();
	new ScriptObject(NotepadUndo);
	Notepad_Txt.setText("");
	Notepad_Txt.notepadFile = "";
	Notepad_Win.setText("Untitled - Notepad");
}
function NFSLSelect() {
	%txt = Notepad_FS_L.getRowTextByID(Notepad_FS_L.getSelectedID());
	Notepad_FS_E.setText(%txt);
	if(Notepad_FS_L.getSelectedID() == Notepad_FS_L.lastselected) Notepad.done(); else Notepad_FS_Input.makeFirstResponder(1);
	Notepad_FS_L.lastselected = Notepad_FS_L.getSelectedID();
	cancel(Notepad_FS_L.stopThisNow);
	Notepad_FS_L.stopThisNow = Notepad_FS_L.schedule(2000,setField,"lastselected","-2");
}
function Notepad_openToolWindow(%mode) {
	%this = Notepad_Tool;
	%textBox1 = Notepad_T_1.getobject(0);
	%textBox2 = Notepad_T_2.getobject(0);
	for(%i=0;%i<%this.getCount();%i++) {
		%obj = %this.getObject(%i);
		if(%obj.getClassName() !$= "GuiSwatchCtrl") continue;
		%swatch = %obj;
		break;
	}
	//%textBox1.setText("");
	//%textBox2.setText("");
	switch(%mode) {
		case 0:
			// Find mode
			%this.setText("Find - Notepad");
			Notepad_T_2.setVisible(0);
			%this.resize(getWord(%this.getPosition(),0),getWord(%this.getPosition(),1),389,95);
			%swatch.resize(getWord(%swatch.getPosition(),0),getWord(%swatch.getPosition(),1),375,59);
			%this.mode = 0;
		case 1:
			// Replace mode
			%this.setText("Replace - Notepad");
			Notepad_T_2.setVisible(1);
			%this.resize(getWord(%this.getPosition(),0),getWord(%this.getPosition(),1),389,125);
			%swatch.resize(getWord(%swatch.getPosition(),0),getWord(%swatch.getPosition(),1),375,88);
			%this.mode = 1;
	}
	%this.setVisible(1);
	%this.getGroup().pushToBack(%this);
}
function Notepad_TXT::findText(%this,%str,%offSetArg) {
	if(%offSetArg $= "") %offSetArg = 0;
	%txt = %this.getValue();
	%txt = strReplace(%txt,"<tag:12>","");
	for(%i=0;%i<getRecordCount(%txt);%i++) {
		%rec = getRecord(%txt,%i);
		for(%j=0;%j<getWordCount(%rec);%j++) {
			%word = getWord(%rec,%j);
			if(strIPos(%word,%str) == -1) continue;
			%offSet++;
			if(%offSet < %offSetArg) continue;
			%rec = setWord(%rec,%j,"<tag:12>"@%word);
			%txtcache = %txt;
			%txtcache = setRecord(%txtcache,%i,%rec);
			%this.setText(%txtcache);
			%this.forceReflow();
			%this.schedule(10,scrollToTag,12);
			return;
		}
	}
}
function Notepad_TXT::replaceText(%this,%str,%replace) {
	%txt = %this.getValue();
	%txt = strReplace(%txt,"<tag:12>","");
	for(%i=0;%i<getRecordCount(%txt);%i++) {
		%rec = getRecord(%txt,%i);
		if(strIPos(%rec,%str) == -1) continue;
		%rec = strReplace(%rec,%str,%replace);
		%txt = setRecord(%txt,%i,%rec);
	}
	Notepad_TXT.setText(%txt);
}
function Notepad_Tool_Done() {
	%this = Notepad_Tool;
	%textBox1 = Notepad_T_1.getobject(0);
	%textBox2 = Notepad_T_2.getobject(0);
	switch(%this.mode) {
		case 0:
			// Find
			Notepad_TXT.findText(%textBox1.getValue(),0);
		case 1:
			// Replace
			Notepad_TXT.replaceText(%textBox1.getValue(),%textBox2.getValue());
	}
}
function Notepad_SaveUndo() {
	%this = Notepad_Txt;
	%so = NotepadUndo;
	if(%this.getValue() $= %so.state[%so.undoCount]) {
		cancel(Notepad.schedulethis);
		Notepad.schedulethis = schedule(4000,0,Notepad_SaveUndo);
		return;
	}
	%so.state[%so.undoCount++] = %this.getValue();
	cancel(Notepad.schedulethis);
	Notepad.schedulethis = schedule(4000,0,Notepad_SaveUndo);
	Notepad_Status.setText("<font:Arial:14>Saved.");
	cancel(Notepad_Status.sch);
	Notepad_Status.sch = Notepad_Status.schedule(500,setText,"<font:Arial:14>Ready");
}
function Notepad_performUndo() {
	%so = NotepadUndo;
	if(%so.state[%so.undoCount-1] $= "") return;
	Notepad_txt.setText(%so.state[%so.undoCount--]);
}
function Notepad_performRedo() {
	%so = NotepadUndo;
	if(%so.state[%so.undoCount+1] $= "") return;
	Notepad_txt.setText(%so.state[%so.undoCount++]);
}
function clientCmdNotepadReceiveData(%t,%str) {
	if(%t !$= $CACHE::Notepad::ClientTicket) return;
	if($Pref::Client::Notepad::HideMLControlChars) %str = stripMLControlChars(%str);
	if($Pref::Client::Notepad::ReplaceTABS) %str = strReplace(%str,"<tab>","   "); else %str = strReplace(%str,"<tab>","");
	Notepad_Txt.addText(%str@"\n",1);
	Notepad_Txt.forceReflow();
}
function clientCmdNotepadDoneFile(%t,%name,%path) {
	if(%t !$= $CACHE::Notepad::ClientTicket) return;
	Notepad_Txt.notepadFile = %path;
	Notepad_Win.setText(%name SPC "- Notepad");
	Notepad_FS.setVisible(0);
}
function clientCmdNotepadReceiveFile(%t,%str,%path) {
	if(%t !$= $CACHE::Notepad::ClientTicket) return;
	%str = StripMLControlChars(%str);
	if(%path !$= "") Notepad_txt.notepadFile = %path;
	if(%path !$= "") Notepad_FS_P.setText(getSubStr(%path,0,strLen(%path)-1));
	for(%i=0;%i<Notepad_FS_L.rowCount();%i++) if(Notepad_FS_L.getRowText(%i) $= %str) return;
	if(fileExt(%str) !$= "") %str = fileName(%str);
	Notepad_FS_E.setText("");
	Notepad_FS_L.addRow(Notepad_FS_L.rowCount()+1,%str);
}
function clientCmdNotepadClear(%t,%s) {
	if(%t !$= $CACHE::Notepad::ClientTicket) return;
	if(%s == 1)
		Notepad_FS_L.clear();
	else
		Notepad_Txt.setText("");
}
function clientCmdCloseNotepadFS(%t) {
	if(%t !$= $CACHE::Notepad::ClientTicket) return;
	Notepad_FS.setVisible(0);
}
function Notepad::saveFile(%this,%a,%clear) {
	if(!isUnlocked()) {
		error("ERROR: saveFile() - Game is in demo mode; cannot save file");
		return;
	}
	%file = Notepad_FS_E.getValue();
	%path = Notepad_FS_P.getValue();
	if(trim(%file) $= "" && Notepad_txt.notepadFile $= "") {
		messageBoxNotepadOk("Error - Notepad","Please enter a file name to save this as.");
		return "Save";
	}
	if(%path !$= "") %path = %path@"/";
	// server
	if(!$Pref::Client::NotepadDisableServerComm && isObject(ServerConnection) && $CACHE::Notepad::IsAdmin) {
		if(fileExt(%file) $= "" || %file $= "..") {
			Notepad_FS.loadFiles(%path@%file);
			Notepad.schedule(300,setField,saveorload,"Save");
			return;
		}
		if(%file $= "")
			%file = Notepad_Txt.notepadFile;
		else 
			%file = %path@%file;		

		%txt = Notepad_Txt.getValue();
		commandToServer('NotepadSave',"START",%file);
		for(%i=0;%i<getRecordCount(%txt);%i++) commandToServer('NotepadSave',getField(%txt,%i));
		commandToServer('NotepadSave',"END");
		return;
	}
	// end
	if(%file $= "..") {
		if(trim(%path) $= "") return;
		%patha = strReplace(%path,"/",""TAB"");
		for(%i=0;%i<getFieldCount(%patha)-1;%i++) %pathb = %pathb@"/"@getField(%patha,%i);
		if(trim(%pathb) !$= "") %pathb = getSubStr(%pathb,1,strLen(%pathb)-1);
		Notepad_FS_P.setText(%pathb);
		Notepad_FS_L.clear();
		Notepad_FS_E.setValue("");
		if(!$Pref::Client::NotepadDisableServerComm && isObject(ServerConnection) && $CACHE::Notepad::IsAdmin)
			commandToServer('NotepadloadFiles',%pathb);
		else
			Notepad_FS.loadFiles(%pathb);

		return "Save";

	}
	if(fileExt(%file) $= "") {
		Notepad_FS_P.setText(%file);
		Notepad_FS_L.clear();
		Notepad_FS_E.setValue("");
		Notepad_FS.loadFiles(%file);
		Notepad_FS_E.setValue("");
		return "Save";
	}
	if(isFile(%file) && !%a) {
		messageBoxYesNo("Notepad","Do you want to overwrite the existing file?","Notepad.saveFile(1);");
		return;
	}
	%txt = Notepad_Txt.getValue();
	%SO = new FileObject();
	%SO.openForWrite(%path@%file);
	for(%i=0;%i<getRecordCount(%txt);%i++) %SO.writeLine(getRecord(%txt,%i));
	%SO.close();
	%SO.delete();
	Notepad_Txt.notepadFile = %path@%file;
	Notepad_Win.setText(fileName(%file) SPC "- Notepad");
	Notepad_FS.setVisible(0);
	if(%clear) Notepad_Clear(1);
}
function Notepad::openFile(%this) {
	%file = Notepad_FS_E.getValue();
	%path = Notepad_FS_P.getValue();
	if(%path !$= "") %path = %path@"/";
	if(%file $= "..") {
		if(trim(%path) $= "") return;
		%patha = strReplace(%path,"/",""TAB"");
		for(%i=0;%i<getFieldCount(%patha)-1;%i++) %pathb = %pathb@"/"@getField(%patha,%i);
		if(trim(%pathb) !$= "") %pathb = getSubStr(%pathb,1,strLen(%pathb)-1);
		Notepad_FS_P.setText(%pathb);
		Notepad_FS_L.clear();
		Notepad_FS_E.setValue("");
		if(!$Pref::Client::NotepadDisableServerComm && isObject(ServerConnection) && $CACHE::Notepad::IsAdmin)
			commandToServer('NotepadloadFiles',%pathb);
		else
			Notepad_FS.loadFiles(%pathb);

		return;

	}
	if(fileExt(%file) $= "") {
		Notepad_FS_P.setText(%file);
		Notepad_FS_L.clear();
		Notepad_FS_E.setValue("");
		Notepad_FS.loadFiles(%file);
		Notepad_FS_E.setValue("");
		return;
	}
	if(!$Pref::Client::NotepadDisableServerComm && isObject(ServerConnection) && $CACHE::Notepad::IsAdmin) {
		if(stripos($CACHE::Notepad::RecentFiles,%path@%file) == -1) {
			$CACHE::Notepad::RecentFiles = %path@%file NL $CACHE::Notepad::RecentFiles;
			canvas.popDialog(Notepad);
			canvas.pushDialog(Notepad);
		}
		commandToServer('NotepadopenFile',%path@%file);
		return;
	}
	if(!isFile(%path@%file)) {
		messageBoxNotepadOk("Error","The file you requested does not exist.");
		return;
	}
	Notepad_Txt.setText("");
	if(stripos($CACHE::Notepad::RecentFiles,%path@%file) == -1) {
		$CACHE::Notepad::RecentFiles = %path@%file NL $CACHE::Notepad::RecentFiles;
		canvas.popDialog(Notepad);
		canvas.pushDialog(Notepad);
	}
	%SO = new FileObject();
	%SO.openForRead(%path@%file);
	while(!%SO.isEOF()) {
		%strLine = %SO.readLine();
		%strLine = strReplace(%strLine,""TAB"","[[tab]]");
		if($Pref::Client::Notepad::HideMLControlChars) %strLine = stripMLControlChars(%strLine);
		if($Pref::Client::Notepad::ReplaceTABS) %strLine = strReplace(%strLine,"[[tab]]","   ");
		%strLine = strReplace(%strLine,"[[tab]]","");
		Notepad_Txt.addText(%strLine@"\n",1);
		if(Notepad_Txt.isAwake()) Notepad_Txt.forceReflow();
	}
	%SO.close();
	%SO.delete();
	Notepad_Txt.notepadFile = %path@%file;
	Notepad_Win.setText(fileName(%file) SPC "- Notepad");
	Notepad_FS.setVisible(0);
}
function Notepad_FS::loadFiles(%this,%directory) {
	if(trim(%directory) !$= "") if(getSubStr(%directory,strLen(%directory)-1,1) !$= "/") %directory = %directory@"/";
	if(!$Pref::Client::NotepadDisableServerComm && isObject(ServerConnection) && $CACHE::Notepad::IsAdmin) {
		commandToServer('NotepadloadFiles',%directory);
		return;
	}
	if(trim(%directory) !$= "") Notepad_FS_L.addRow(0,"..");
	for(%file = findFirstFile(%directory@"*");%file !$= ""; %file = findNextFile(%directory@"*")) {
		// we're getting folders here, boy
		%found = 0;
		%path = filePath(%file);
		%pathchk = strReplace(%path,%directory,"");
		%pathchk = strReplace(%pathChk,"/",""TAB"");
		if(getFieldCount(%pathchk) != 1) continue;
		for(%i=0;%i<Notepad_FS_L.rowCount();%i++) if(Notepad_FS_L.getRowText(%i) $= %path) %found=1;
		if(%found) continue;
		if(trim(%path) $= "") continue;
		Notepad_FS_L.addRow(Notepad_FS_L.rowCount()+1,%path);
	}
	%allowedFiles = "_.txt_.cs_";
	for(%file = findFirstFile( %directory@"*.*" );%file !$= ""; %file = findNextFile(%directory@"*.*")) {
		if(stripos(%allowedFiles,"_"@fileExt(%file)@"_") == -1) continue;
		if(filePath(%file)@"/" !$= %directory) continue;
		if(fileExt(%file) $= "") continue;
		Notepad_FS_L.addRow(getFileCRC(%file),fileName(%file));
	}
}
function Notepad::openOptionsMenu(%that) {
	%this = Notepad_Opt;
	%this.setVisible(1);
	%that.pushToBack(%this);
	%this.centerY();
	%this.centerX();
	Notepad_loadOptionsGUI();
}
function Notepad_sendEval() {
	if(!$IAmAdmin) {
		Notepad_doNotification("Invalid Rank","You must be a Super Admin to modify server files and execute code.","lock", 0,(!AdminLoginGui.isAwake() ? "canvas.pushDialog(AdminLoginGui);" : ""));
		return;
	}
	%txt = Notepad_Txt.getValue();
	commandToServer('NotepadEval',"START");
	for(%i=0;%i<getRecordCount(%txt);%i++) commandToServer('NotepadEval',getRecord(%txt,%i));
	commandToServer('NotepadEval',"END");
}
function Notepad_setTheme(%themename) {
	%this = Notepad;
	if(%themeName !$= "Default" && !isObject("NotepadWindowTheme"@%themename)) {
		error("ERROR: Notepad_setTheme() - Theme does not exist.");
		cancel($Notepad::CacheSchedule);
		$Notepad::CacheSchedule = schedule(3,0,Notepad_setTheme,"Default");
		return;
	}
	%themeObj = "NotepadWindowTheme"@%themename;
	%notepadButton = "add-ons/system_notepad/close";
	if(%themeName $= "Default")
		%themeObj = NotepadWindowProfile;

	GuiNotepadBarProfile.fillColor = %themeObj.notepadFill;
	%pic = %themeObj.notepadButton;
	if(isFile(%pic@"_n.png")) %notepadButton = %pic;
	%cc = Notepad.isAwake();
	if(%cc) canvas.popDialog(notePad);
	for(%i=0;%i<%this.getCount();%i++) {
		%obj = %this.getObject(%i);
		if(%obj.getClassName() !$= "GuiWindowCtrl") continue;
		%obj.setProfile(%themeObj);
		for(%j=0;%j<%obj.getCount();%j++) {
			%btn = %obj.getObject(%j);
			if(%btn.getClassName() !$= "GuiBitmapButtonCtrl") continue;
			%btn.bitmap = %notepadButton;
			break;
		}
	}
	for(%i=0;%i<notepadThemeGui.getCount();%i++) {
		%obj = notepadThemeGui.getObject(%i);
		if(%obj.getClassName() !$= "GuiWindowCtrl") continue;
		%obj.setProfile(%themeObj);
		for(%j=0;%j<%obj.getCount();%j++) {
			%btn = %obj.getObject(%j);
			if(%btn.getClassName() !$= "GuiBitmapButtonCtrl") continue;
			%btn.bitmap = %notepadButton;
			break;
		}
	}
	if(%cc) {
		cancel($Notepad::CacheSchedule);
		$Notepad::CacheSchedule = canvas.schedule(10,pushDialog,notePad);
	}
}
function initNotepadThemes() {
	if(!isFile("config/client/NotepadThemes/readme.txt")) {
		%SO = new FileObject();
		%SO.openForWrite("config/client/NotepadThemes/readme.txt");
		%SO.writeLine("This is the folder to store your notepad themes in.");
		%SO.writeLine("Simply create a new folder in here with the theme name,");
		%SO.writeLine("and then add a 'info.txt' file in that folder, along with");
		%SO.writeLine("the window bitmap, and the close button bitmaps.");
		%SO.writeLine("The info.txt file must be formatted as follows:");
		%SO.writeLine("");
		%SO.writeLine("Theme name");
		%SO.writeLine("Theme bitmap location (ex. \"config/client/NotepadThemes/mytheme/windowBitmap\")");
		%SO.writeLine("");
		%SO.writeLine("The buttons bitmaps must be named 'close_n.png', 'close_h.png', and 'close_d.png' with a size of 45x26.");
		%SO.writeLine("If the theme bitmap location is left blank, then the loader will look for windowBitmap.png in the theme's folder.");
		%SO.close();
		%SO.delete();
	}
	if(isObject(NotepadThemeGroup)) NotepadThemeGroup.delete();
	%so = new ScriptGroup(NotepadThemeGroup);
	%this = guiDataGroup;
	%gotObj = 0;
	for(%i=0;%i<%this.getCount();%i++) {
		%obj = %this.getObject(%i);
		if(stripos(%obj.getName(),"notepadwindowtheme") != 0) continue;
		%gotObj++;
		if(%obj.notepadFill $= "") %obj.notepadFill = "222 222 222 255";
		%so.add(%obj);
	}
	%poss = "s";
	if(%gotObj == 1) %poss = "";
	echo("Got "@%gotObj@" theme"@%poss@".");
}
function NotepadThemes_initLoader() {
	%SO = new FileObject();
	%findPath = "config/client/NotepadThemes/";
	for(%file = findFirstFile(%findPath@"*");%file !$= ""; %file = findNextFile(%findPath@"*")) {
		%file = strReplace(%file,%findPath,"");
		%themeName = filePath(%file);
		%themeName = strReplace(%themeName," ","");
		%themeBitmap = %findPath@%themeName@"/windowBitmap";
		%fontFace = "Arial";
		if(stripos(%executed,"_"@%themeName@"_") != -1) continue;
		if(%themeName $= %findPath) continue;
		%executed = %executed@"_"@%themeName@"_";
		if(isFile(%findPath@%themeName@"/info.txt")) {
			%line = 0;
			%argCount = 0;
			%SO.openForRead(%findPath@%themeName@"/info.txt");
			while(!%SO.isEOF()) {
				%line++;
				%strLine = %SO.readLine();
				switch(%line) {
					case 1:
						// Theme Name
						%themeNameInt = %strLine;
					case 2:
						// Theme bitmap
						%themeBitmap = %strLine;
					case 3:
						// Font Name
						%fontFace = %strLine;
					default:
						if(trim(%strLine) !$= "") {
							%argname[%argCount++] = firstWord(%strLine);
							%argval[%argCount] = removeWord(%strLine,firstWord(%strLine));
						}
				}
			}
			%SO.close();
			if(trim(%themeNameInt) $= "") {
				error("ERROR: NotepadThemes_initLoader() - Theme "@%themeName@" has an invalid theme name.");
				continue;
			}
			%themeButtonBitmap = "add-ons/system_notepad/close";
			if(trim(%fontFace) $= "") %fontFace = "Arial";
			if(trim(%themeBitmap) $= "") %themeBitmap = "add-ons/system_notepad/notepadWindow";
			if(isFile(%findPath@%themeName@"/close_n.png")) %themeButtonBitmap = %findPath@%themeName@"/close";
			%objName = "NotepadWindowTheme"@%themeName;
			if(!isObject(%objName)) {
				%themeObjNew = new GuiControlProfile(%objName) {
					notepadButton = %themeButtonBitmap;
					notepadName = %themeNameInt;
					notepadFill = "222 222 222 255";
					tab = "0";
					canKeyFocus = "0";
					mouseOverSelected = "0";
					modal = "1";
					opaque = "1";
					fillColor = "255 255 255 255";
					fillColorHL = "200 200 200 255";
					fillColorNA = "200 200 200 255";
					border = "2";
					borderThickness = "7";
					borderColor = "0 0 0 255";
					borderColorHL = "128 128 128 255";
					borderColorNA = "64 64 64 255";
					fontType = %fontFace;
					fontSize = "20";
					fontColors[0] = "50 50 50 255";
					fontColors[1] = "255 255 255 255";
					fontColors[2] = "0 0 0 255";
					fontColors[3] = "200 200 200 255";
					fontColors[4] = "0 0 204 255";
					fontColors[5] = "85 26 139 255";
					fontColors[6] = "0 0 0 0";
					fontColors[7] = "0 0 0 0";
					fontColors[8] = "0 0 0 0";
					fontColors[9] = "0 0 0 0";
					fontColor = "50 50 50 255";
					fontColorHL = "255 255 255 255";
					fontColorNA = "0 0 0 255";
					fontColorSEL = "200 200 200 255";
					fontColorLink = "0 0 204 255";
					fontColorLinkHL = "85 26 139 255";
					doFontOutline = "0";
					fontOutlineColor = "255 255 255 255";
					justify = "left";
					textOffset = "30 3";
					autoSizeWidth = "0";
					autoSizeHeight = "0";
					returnTab = "0";
					numbersOnly = "0";
					cursorColor = "0 0 0 255";
					bitmap = %themeBitmap;
						text = "GuiWindowCtrl .";
						hasBitmapArray = "1";
				};
				for(%i=0;%i<%argCount;%i++) eval(%themeObjNew@"."@%argName[%i]@" = \""@%argval[%i]@"\";");
				NotepadThemeGroup.add(%themeObjNew);
			}
		}
	}
	%SO.delete();
	if($CACHE::NotepadDoneInit) {
		$CACHE::NotepadDoneInit = 0;
		if(Notepad.isAwake()) {
			canvas.popDialog(Notepad);
			canvas.pushDialog(Notepad);
		}
		messageBoxNotepadOk("Success - Notepad","Re-initialization finished!");
	}
}
// Theme Gui Creator
function NotepadThemeGui::onWake(%this) {
	for(%i=0;%i<%this.getCount();%i++) {
		%obj = %this.getObject(%i);
		if(%obj.getClassName() !$= "GuiWindowCtrl") continue;
		%obj.centerY();
		%obj.centerX();
	}
	%msg = "Welcome to the notepad theme creator. Here you can create custom themes with bitmaps! Here's a rundown on the GUI:\n\n";
	%msg = %msg @ "Theme Name - The name of your theme. This will be the folder name located in 'config/client/NotepadThemes/'.\n\n";
	%msg = %msg @ "Font Type - This is the font name that you want the titlebar of the window to use.\n\n";
	%msg = %msg @ "Bitmap array - The window\'s bitmap array. It must end with '.png'.\n\n";
	%msg = %msg @ "Close btn bitmap - The window\'s close button. The recommended size is 45x26 for all three bitmaps.";
	if(!$CACHE::OpenedThemeGuiBefore) {
		$CACHE::OpenedThemeGuiBefore = 1;
		messageBoxNotepadOk("Info - Notepad",%msg);
	}
	NotepadThemeGui_Name.setText("");
	NotepadThemeGui_Font.setText("");
	NotepadThemeGui_Bitmap.setText("");
	NotepadThemeGui_BitmapShow.setBitmap("add-ons/system_notepad/notfound.png");
	NotepadThemeGui_BitmapShowBtn_n.setBitmap("add-ons/system_notepad/Notfound_n.png");
	NotepadThemeGui_BitmapShowBtn_h.setBitmap("add-ons/system_notepad/Notfound_n.png");
	NotepadThemeGui_BitmapShowBtn_d.setBitmap("add-ons/system_notepad/Notfound_n.png");
	NotepadThemeGui_BitmapBtn_All.setBitmap("add-ons/system_notepad/Notfound");
}
function NotepadThemeGuiTypeBitmap() {
	%txt = NotepadThemeGui_Bitmap.getValue();
	%this = NotepadThemeGui_BitmapShow;
	if(!isFile(%txt) && %this.bitmap !$= "add-ons/system_notepad/notfound.png") {
		%this.setBitmap("add-ons/system_notepad/notfound.png");
		return;
	} else if(!isFile(%txt)) return;
	%this.setBitmap(%txt);
}
function NotepadThemeGuiTypeButtonBitmap() {
	%txt = NotepadThemeGui_BtnBitmap.getValue();
	%this_n = NotepadThemeGui_BitmapShowBtn_n;
	%this_h = NotepadThemeGui_BitmapShowBtn_h;
	%this_d = NotepadThemeGui_BitmapShowBtn_d;
	%this_all = NotepadThemeGui_BitmapBtn_All;
	if(!isFile(%txt@"_n.png") && %this.bitmap !$= "add-ons/system_notepad/Notfound_n.png")
		%this_n.setBitmap("add-ons/system_notepad/Notfound_n.png");
	else if(isFile(%txt@"_n.png")) %this_n.setBitmap(%txt@"_n.png");
	if(!isFile(%txt@"_h.png") && %this.bitmap !$= "add-ons/system_notepad/Notfound_n.png")
		%this_h.setBitmap("add-ons/system_notepad/Notfound_n.png");
	else if(isFile(%txt@"_h.png")) %this_h.setBitmap(%txt@"_h.png");
	if(!isFile(%txt@"_d.png") && %this.bitmap !$= "add-ons/system_notepad/Notfound_n.png")
		%this_d.setBitmap("add-ons/system_notepad/Notfound_n.png");
	else if(isFile(%txt@"_d.png")) %this_d.setBitmap(%txt@"_d.png");
	if(!isFile(%txt@"_n.png") && %this.bitmap !$= "add-ons/system_notepad/Notfound")
		%this_all.setBitmap("add-ons/system_notepad/Notfound");
	else if(isFile(%txt@"_n.png")) %this_all.setBitmap(%txt);
}
function NotepadThemeGui_createTheme() {
	%name =   NotepadThemeGui_Name.getValue();
	%font =   NotepadThemeGui_Font.getValue();
	%bitmap = NotepadThemeGui_Bitmap.getValue();
	if(%err $= "" && trim(%name) $= "") %err = "The theme name is required.";
	if(%err $= "" && trim(%font) $= "") %err = "The font type is required.";
	if(%err $= "" && !isFile(%bitmap)) %err = "Bitmap file does not exist.";
	if(%err !$= "") {
		messageBoxNotepadOk("Error - Notepad","Could not create theme. Error:\n\n\""@%err@"\"");
		return;
	}
	%SO = new FileObject();
	%SO.openForWrite("config/client/NotepadThemes/"@%name@"/info.txt");
	%SO.writeLine(%name);
	%SO.writeLine("config/client/NotepadThemes/"@%name@"/windowBlock.png");
	%SO.writeLine(%font);
	%SO.close();
	%SO.delete();
	fileCopy(%bitmap,"config/client/NotepadThemes/"@%name@"/windowBlock.png");
	canvas.popDialog(NotepadThemeGui);
	messageBoxNotepadOk("Success - Notepad","Theme created, re-initializing themes...");
	$CACHE::NotepadDoneInit = 1;
	cancel($CACHE::NotepadSchRe);
	$CACHE::NotepadSchRe = schedule(100,0,NotepadThemes_initLoader);
}
// Document storing (for later use)
// NotepadStDoc = NotepadStoreDocument
function NotepadStDoc_storeDoc(%confirm) {
	%this = Notepad_Txt;
	if(%confirm != 1 && %confirm != 2) {
		messageBoxYesNo("Confirmation - Notepad","Are you sure you want to store this document?","NotepadStDoc_storeDoc(1);");
		return;
	}
	%SOName = "NotepadDocument"@getStringCRC(%this.notepadFile);
	if(isObject(%SOName) && %confirm != 2) {
		messageBoxYesNo("Confirmation - Notepad","You already have a document stored with this file name. Do you want to overwrite it?","NotepadStDoc_storeDoc(2);");
		return;
	}
	if(isObject(%SOName)) %SOName.delete();
	%SO = new ScriptObject(%SOName);
	%SO.docName = fileBase(%this.notepadFile);
	%SO.docFile = %this.notepadFile;
	%SO.docDate = getWord(getDateTime(),1);
	%SO.docModifier = "C";
	if(isObject(ServerConnection) && $CACHE::Notepad::IsAdmin && !$Pref::Client::NotepadDisableServerComm) %SO.docModifier = "S";
	NotepadDocGroup.add(%SO);
	for(%i=0;%i<getRecordCount(%this.getValue());%i++) %SO.line[%i] = getRecord(%this.getValue(),%i);
	%SO.lineCount = getRecordCount(%this.getValue());
	canvas.popDialog(Notepad);
	canvas.pushDialog(Notepad);
	schedule(2,0,messageBoxNotepadOk,"Success - Notepad","Document successfully stored.");
}
function NotepadStDoc_getDoc(%docID,%confirm) {
	if(!isObject(%docID)) {
		messageBoxNotepadOk("Error - Notepad","The document you requested does not exist in memory.");
		return;
	}
	if(!%confirm) {
		messageBoxYesNo("Confirmation - Notepad","Are you sure you want to get this document?\nThis will erase the current contents in notepad.","NotepadStDoc_getDoc(\""@%docID@"\",1);");
		return;
	}
	Notepad_txt.setText("");
	for(%i=0;%i<%docID.lineCount;%i++) {
		%strLine = %docID.line[%i];
		%strLine = strReplace(%strLine,""TAB"","[[tab]]");
		if($Pref::Client::Notepad::HideMLControlChars) %strLine = stripMLControlChars(%strLine);
		if($Pref::Client::Notepad::ReplaceTABS) %strLine = strReplace(%strLine,"[[tab]]","   ");
		%strLine = strReplace(%strLine,"[[tab]]","");
		Notepad_Txt.addText(%strLine@"\n",1);
		if(Notepad_Txt.isAwake()) Notepad_Txt.forceReflow();
	}
	Notepad_Txt.notepadFile = %docID.docFile;
	Notepad_Win.setText(fileName(%docID.docFile) SPC "- Notepad");
	messageBoxNotepadOk("Success - Notepad","Loaded document.");
}
// Message box support
function messageBoxNotepadOk(%title,%text) {
	MBOKFrame.setProfile($CACHE::NotepadThemeText);
	messageBoxOk(%title,%text);
}
// Options
$Notepad::Options = "COMMENT -- Client/Server --	$Pref::Client::NotepadDisableServerComm Only show my files	COMMENT -- Client --	$Pref::Client::Notepad::HideMLControlChars Hide ML markup text	$Pref::Client::Notepad::ReplaceTABS Replace TAB indents with three spaces	$Pref::Client::Notepad::AllowNotepadCMD Allow /notepad on any server	$Pref::Client::Notepad::AllowNotepadConsole Allow 'notepad' command in console";
function Notepad_loadOptionsGUI() {
	%a = $Notepad::Options;
	%this = Notepad_Options;
	if(%a $= "") return;
	%this.clear();
	%this.addRow(-2,"\c0I/O\tPreference / category description\tCOMMENT",0);
	%this.addRow(-2,"\c0______\t_________________________________________\tCOMMENT",1);
	for(%i=0;%i<getFieldCount(%a);%i++) {
		%data = getField(%a,%i);
		if(firstWord(%data) $= "COMMENT") {
			%this.addRow(%this.rowCount()+1,"\c2" TAB removeWord(%data,0) TAB "COMMENT");
			continue;
		}
		%pref = firstWord(%data);
		%desc = removeWord(%data,0);
		eval("$CACHE::NotepadOptionsC = "@%pref@";");
		%this.addRow(%this.rowCount()+1,($CACHE::NotepadOptionsC ? "\c5ON\c0" : "\c4OFF\c0") TAB %desc TAB %pref TAB $CACHE::NotepadOptionsC);
	}
	deleteVariables("$CACHE::NotepadOptionsC");
}
function Notepad_OptionsonSelect() {
	%this = Notepad_Options;
	%cellid = %this.getSelectedID();
	%text = %this.getRowTextByID(%cellid);
	%pref = getField(%text,2);
	if(%pref $= "COMMENT") {
		%this.clearSelection();
		return;
	}
	// Let's make sure we're double clicking here
	%doubleClickCache = %this.lastSelected;
	%this.lastSelected = %cellid;
	cancel(%this.stopDoubleClick);
	%this.stopDoubleClick = %this.schedule(1000,setField,lastSelected,-1);
	if(%doubleClickCache != %cellid) return;
	// START
	%prefVal = getField(%text,3);
	%textR = setField(%text,0,"\c4OFF\c0");
	%textR = setField(%textR,3,"0");
	if(%prefVal)
		eval(%pref@" = \"0\";");
	else {
		%textR = setField(%text,0,"\c5ON\c0");
		%textR = setField(%textR,3,"1");
		eval(%pref@" = \"1\";");
	}
	%this.setRowByID(%cellid,%textR);
	%this.setField(lastSelected,-1);
	%this.clearSelection();
}
// Server > Client's GUI
function clientCmdNotepadHandshake(%isAdmin) {
	$CACHE::Notepad::IsAdmin = %isAdmin;
	$CACHE::Notepad::ClientTicket = getNonsense();
	commandToServer('NotepadHS',$CACHE::Notepad::ClientTicket);
}
function Notepad_FS::deleteFile(%this,%confirm) {
	%text = Notepad_FS_L.getRowTextByID(Notepad_FS_L.getselectedid());
	%file = Notepad_FS_P.getValue()@"/"@%text;
	if(%text $= "" || fileExt(%text) $= "") {
		messageBoxOk("Error - Notepad","You can only delete files.");
		return;
	}
	if(!%confirm) {
		messageBoxYesNo("Confirmation - Notepad","Are you sure you wish to delete this file?","Notepad_FS.deleteFile(1);");
		return;
	}
	if(!$Pref::Client::NotepadDisableServerComm && isObject(ServerConnection) && $CACHE::Notepad::IsAdmin) {
		commandToServer('NotepadDeleteFile',%file);
		return;
	}
	if(!isFile(%file)) {
		messageBoxOk("Error - Notepad","This file does not exist and cannot be deleted.");
		return;
	}
	fileDelete(%file);
	setModPaths(getModPaths());
	Notepad_FS_L.clear();
	Notepad_FS_E.setValue("");
	Notepad_FS.loadFiles(Notepad_FS_P.getValue());
	messageBoxOk("Notepad","Deleted file.");
}
function Notepad_FS_Input_doInterpretation() {
	%txt = Notepad_FS_Input.getValue();
	%val = strReplace(%txt,"!","");
	switch$(%txt) {
		case "!d":
			Notepad_FS.deleteFile();
		case "!f":
			Notepad_FS_E.makeFirstResponder(1);
		case "!p":
			Notepad_FS_P.makeFirstResponder(1);
		case "!?":
			%controls = "Keybinds you can use in the file selector:<br>";
			%controls = %controls @ "d - Delete the selected file.<br>";
			%controls = %controls @ "p - Edit the file path manually.<br>";
			%controls = %controls @ "f - Edit the file name manually.<br>";
			%controls = %controls @ "? - Shows the help.<br>";
			%controls = %controls @ "backspace - Delete the selected file.<br>";
			messageBoxOk("Notepad",%controls);
		case "!":
			// Do nothing.
		case "":
			Notepad_FS.deleteFile();
	}
	Notepad_FS_Input.setValue("!");
}
// Animation
function Notepad_doNotification(%title,%text,%image,%sticky,%callback) {
	if(!isObject(GlassNotificationManager)) return;
	//Notepad_doNotification("Invalid Rank","You must be a Super Admin to modify server files and execute code.","lock", 0,(!AdminLoginGui.isAwake() ? "canvas.pushDialog(AdminLoginGui);" : ""));
	GlassNotificationManager::newNotification(%title,%text,%image,%sticky,%callback);
}
function Notepad_setFont(%fontName,%fontSize) {
	NotepadTextEdit.fontType = %fontName;
	NotepadTextEdit.fontSize = %fontSize;
	NotepadTextEdit.updateFont();
	if(!Notepad.isAwake()) return;
	canvas.popDialog(Notepad);
	canvas.pushDialog(Notepad);
}
//NotepadFont_Sample
//NotepadFont_SizeList
//NotepadFont_StyleList
//NotepadFont_FontList
if($Pref::Client::Notepad::DefaultStyle $= "") $Pref::Client::Notepad::DefaultStyle = "Regular";
if($Pref::Client::Notepad::DefaultFont $= "") $Pref::Client::Notepad::DefaultFont = "Arial";
if($Pref::Client::Notepad::DefaultSize $= "") $Pref::Client::Notepad::DefaultSize = 14;
$CACHE::Notepad::Fonts = "Arial	Comic Sans MS	Verdana	Palatino Linotype	Impact	Consolas	Constantia	Courier New";
$CACHE::Notepad::Sizes = "8 9 10 11 12 14 16 18 20 22 24 26 28 30 32 34 36 38 40";
$CACHE::Notepad::Styles = "Regular	Italic	Bold";
function Notepad_initFontWindow() {
	%sample = NotepadFont_Sample;
	%size = NotepadFont_SizeList;
	%style = NotepadFont_StyleList;
	%font = NotepadFont_FontList;
	%sample.setText("<just:center><color:000000>AaBbYyZz");
	%size.clear();
	%style.clear();
	%font.clear();
	for(%i=0;%i<getFieldCount($CACHE::Notepad::Fonts);%i++) {
		%fonttxt = getField($CACHE::Notepad::Fonts,%i);
		%font.addRow(%i+1,%fonttxt);
	}
	for(%i=0;%i<getWordCount($CACHE::Notepad::Sizes);%i++) {
		%sizetxt = getWord($CACHE::Notepad::Sizes,%i);
		%size.addRow(%i+1,%sizetxt);
	}
	for(%i=0;%i<getWordCount($CACHE::Notepad::Styles);%i++) {
		%styletxt = getWord($CACHE::Notepad::Styles,%i);
		%style.addRow(%i+1,%styletxt);
	}
	%style.setSelectedRow(%style.findTextIndex($Pref::Client::Notepad::DefaultStyle));
	%font.setSelectedRow(%font.findTextIndex($Pref::Client::Notepad::DefaultFont));
	%size.setSelectedRow(%size.findTextIndex($Pref::Client::Notepad::DefaultSize));
	NotepadFont_UpdateFontSample();
	Notepad_Font.centerX();
	Notepad_Font.centerY();
	Notepad_Font.setVisible(1);
	Notepad.pushToBack(Notepad_Font);
}
function NotepadFont_UpdateFontSample() {
	%sample = NotepadFont_Sample;
	%size = NotepadFont_SizeList;
	%style = NotepadFont_StyleList;
	%font = NotepadFont_FontList;
	%fontType = %font.getRowTextByID(%font.getSelectedID());
	%styleType = %style.getRowTextByID(%style.getSelectedID());
	if(%styleType $= "Regular") 
		%styleType = "";
	else
		%styleType = " "@%styleType;

	%fontType = %fontType @ %styleType;
	%text = "<just:center><font:"@%fontType@":"@%size.getRowTextByID(%size.getSelectedID())@">AaBbYyZz";
	%sample.setText(%text);
}
function NotepadFont_saveChanges() {
	%size = NotepadFont_SizeList;
	%style = NotepadFont_StyleList;
	%font = NotepadFont_FontList;
	%fontType = %font.getRowTextByID(%font.getSelectedID());
	$Pref::Client::Notepad::DefaultFont = %fontType;
	%styleType = %style.getRowTextByID(%style.getSelectedID());
	%sizeType = %size.getRowTextByID(%size.getSelectedID());
	%fontType = %fontType @ %styleType;
	$Pref::Client::Notepad::DefaultStyle = %styleType;
	$Pref::Client::Notepad::DefaultSize = %sizeType;
	Notepad_setFont(%fontType,%sizeType);
	Notepad_Font.setVisible(0);
}